/*

Network Library
Copyright (c) 2001 Nicolas Roard

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef __LIBNET_H__
#define __LIBNET_H__

#include <iostream>
#include <string>
#include <errno.h>
#include <netdb.h>
#include <sys/time.h>
#include <netinet/ip.h>
#include <netinet/ip_icmp.h>

#include <sys/socket.h>
#include <sys/unistd.h>
#include <stdio.h>
#include <stdlib.h>

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
//#include <sstream>
#include <strstream>

const int SOCKET_CLOSED = -1;
const int ERREUR = -1;

#define ostringstream ostrstream

class EXCEPTION
{
    public:
	EXCEPTION (string str) 
	{ 
	    ostringstream toto;
	    toto << "Erreur " << str << " [errno : " << strerror(errno) << "(" << errno << ")] "; 
	    mStr = toto.str();
	}
	string getString () { return mStr; }
    private:
	string mStr;
};

class Socket 
{
	public:
	    Socket (int port, int type = SOCK_STREAM, int protocole = 0);
	    Socket (int sd, struct sockaddr_in adr);
	    Socket (const Socket& s);
	    Socket ();
	    string read (int nb);
	    string readall ();
	    string readln ();
	    int write (string str);
	    int getSocket ();
	    int getPort ();
	    string getAdresse ();
	    int setOption (int optname, void* optval, int optlen);
	    int setOption (int optname, int option = 1);
	    int getProto  (string name);
	    int close ();
	    
	protected:
	    int mSocket;
	    struct sockaddr_in mAdresse;
};

class SocketClient : public Socket
{
    public:
	SocketClient (string adr, int port, int type = SOCK_STREAM, int protocole = 0);
	SocketClient (const SocketClient& sd);
	SocketClient ();
	int getPortDest ();
	string getAdresseDest ();
    protected:
	struct sockaddr_in mAdresseDest;
};

class SocketServeur : public Socket
{
    public:
	SocketServeur (int port, bool opt, int type = SOCK_STREAM, int protocole = 0);
	SocketServeur (int port, int type = SOCK_STREAM, int protocole = 0);
	SocketServeur (Socket sd, bool opt);
	SocketServeur ();
	Socket accept ();
    
};

#endif
