/*
Copyright (c) 2001 Nicolas Roard

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include "libnet.h"

Socket::Socket (int port, int type, int protocole)
{
    mSocket = ::socket (AF_INET,type,protocole);
    if (mSocket == ERREUR) throw EXCEPTION ("cration socket");

    int longueur = sizeof (struct sockaddr_in);

    bzero ((char*) &mAdresse, longueur);
    mAdresse.sin_family = AF_INET;
    mAdresse.sin_port = htons(port);
}

Socket::Socket (int sd, sockaddr_in adr)
{
    mSocket = sd;
    mAdresse = adr;
}

Socket::Socket (const Socket &sd)
{
    mSocket = sd.mSocket;
    mAdresse = sd.mAdresse;
}

Socket::Socket ()
{
    mSocket = SOCKET_CLOSED;
}

int Socket::getSocket()
{
    return mSocket;
}

int Socket::getPort()
{
    int ret = -1;
    if (mSocket != SOCKET_CLOSED)
    ret = ntohs(mAdresse.sin_port);
    return ret;
}

string Socket::getAdresse ()
{
    string ret = "INCONNUE";

    if (mSocket != SOCKET_CLOSED)
    {
	ret = inet_ntoa (mAdresse.sin_addr);
    }
    return ret;
}

int Socket::setOption (int optname, void *optval, int optlen)
{
    int ret = -1;
    ret = ::setsockopt (mSocket, SOL_SOCKET, optname, optval, optlen);
    if (ret == ERREUR) throw EXCEPTION ("setsockopt");
    return ret;
}

int Socket::setOption (int optname, int option = 1)
{
    int on = option;
    int ret = -1;
    ret = ::setsockopt (mSocket, SOL_SOCKET, optname, &on, sizeof(on));
    if (ret == ERREUR) throw EXCEPTION ("setsockopt");
    return ret;
}

int Socket::getProto (string name)
{
    int proto = 0;
    struct protoent temp;

    temp = *getprotobyname (name.c_str());
    proto = temp.p_proto;
    return proto;
}

/*
Buffer NetSocket::read (int nb, int* nbRead)
{
    QString ret = "ERROR";
    if (mERROR == 0)
    {
        char *buf = new char [nb];
        int result = ::read(mSocket, buf, nb);
        WARN ("erreur de lecture sur la socket", result > 0)
        if (result == -1) mERROR = -1;
        ret = QString(buf);
        delete [] buf;
        buf = NULL;
        if (nbRead != 0)
        {
            *nbRead = result;
        }
    }
    return Buffer(ret);
}
*/

/*
int NetSocket::read (void* buf, int nb)
{
    int ret = -1;
    if (mERROR == 0)
    {
        ret = ::read (mSocket, buf, nb);
        if (ret == -1) mERROR = -1;
    }
    return ret;
}
*/

/*
int NetSocket::write (void* buf, int nb)
{
    //MSG ("on envoie " << nb << " octets sur la socket " << mSocket);
    int ret = -1;
    if (mERROR == 0)
    {
        ret = ::write (mSocket, buf, nb);
        if (ret == -1) mERROR = -1;
    }
    return ret;
}
*/

string Socket::readall()
{
    int nb=4096;
    char* buf = new char[nb];
    int result = 0;
    int gres = 0;
    bool fin = false;
    string ret;
    while (!fin)
    {
	result = ::read(mSocket, buf, nb);
	gres += result;
	if (result > 0)
	{
	    //ret += buf;
	    for (int i = 0; i< result; i++)
	    {
		ret += buf[i];
		//ret.push_back (buf[i]);
	    }
	}
	else
	{
	    if (result == 0)
	    {
		fin = true;
	    }
	    else
	    {
		throw EXCEPTION("read");
	    }
	}
    }
    delete [] buf;
    buf = NULL;
    return ret;
}

/*
Buffer NetSocket::readall ()
{
    Buffer retour;
    int nb = 4096;
    char* buf = new char [nb];
    bool fin = false;
    int result = 0;
    int taille = 0;
    if (mERROR == 0)
    {
        while (!fin)
        {
            //            MSG("TAILLE LUE : " << taille);
            result = ::read(mSocket, buf, nb);
            if (result == -1) mERROR = -1;
            if (result > 0) // si pas d'erreurs de lecture
            {
                taille += result;
                retour.add(buf,result);
                //		MSG("TAILLE MISE DANS RETOUR ==> " << retour.getSize());
            }
            else
            {
                if (result == 0)
                {
                    fin = true;
                }
                else if (result == -1)
                {
                    fin = true;
                }// sinon on stoppe
            }
        }
    }
    delete [] buf;
    buf = NULL;
    //    MSG("Fin readall ");
    return retour;
}
*/

/*
QString NetSocket::readln ()
{
    QString retour = "";
    char car;
    bool fin = false;
    bool LF = false;
    bool CR = false;
    int MAX = 512;  // valeur max, au cas ou
    int n = 0;
    int result = 0;
    if (mERROR == 0)
    {
        while (!fin)
        {
            result = ::read(mSocket, &car, 1);
            if (result == -1) mERROR = -1;
            if (result > 0) // si pas d'erreurs de lecture
            {
                if (car == '\r')
                {
                    //CR = true; // y'a un CR
                    //retour += car;
                }
                else if (car == '\n') // LF, on stoppe
                {
                    fin = true;
                    //if (CR) // si il y eu un CR
                    //{
                    //    retour = retour.left (retour.length() -1);
                    // }
                }
                else
                {
                    CR = false; // on ne prend plus en compte le CR
                    retour += car;
                }
                n++;
            }
            else
            {
                retour = '\0';
                if (result == 0)
                {
                    fin = true;
                }
                else if (result == -1)
                {
                    fin = true;
                }// sinon on stoppe
            }
            if (n == MAX) fin = true;
        }
    }
    return retour;
}
*/

/*
int NetSocket::write (QString str)
{
    //MSG ("on envoie " << str.length() << " octets sur la socket " << mSocket);
    int ret = -1;
    if (mERROR == 0)
    {
        ret = ::write (mSocket, str.data(), str.length());
        if (ret == -1) mERROR = -1;
        WARN ("NetSocket::write ==> on a pas envoy suffisamment d'octets : " << ret << "/" << str.length(), ret == (int) str.length())
    }
    return ret;
}
*/

/*
int NetSocket::writeln (QString str)
{
    int ret = -1;
    if (mERROR == 0)
    {
        str += "\n";
        //MSG ("on envoie " << str.length() << " octets sur la socket " << mSocket);
        ret = ::write (mSocket, str.data(), str.length());
        if (ret == -1) mERROR = -1;
        WARN ("NetSocket::write ==> on a pas envoy suffisamment d'octets : " << ret << "/" << str.length(), ret == (int) str.length())
    }
    return ret;
}
*/

string Socket::read (int nb)
{
    char buffer [nb];
    int ret = -1;
    ret = ::read (mSocket, buffer, nb);
    if (ret == ERREUR) throw EXCEPTION ("lecture dans la socket");
    return string (buffer);
}

string Socket::readln ()
{
	char buf;
	string sret;
	bool fin = false;

	while (!fin)
	{
		int ret = ::read (mSocket, &buf, 1);
		if (ret == 1) 
		{
			sret += buf;
			if (buf == '\n')
			{
				fin = true;
			}
		}
		else fin = true;
	}
	return sret;
}

/*

string Socket::readln (int nb)
{
    char buffer [nb];
    int ret = -1;
    bool fin = false;
    
    ret = ::read (mSocket, buffer, nb);
    if (ret == ERREUR) throw EXCEPTION ("lecture dans la socket");
    
    for (int i = 0; (i < nb) && (!fin) ; i++)
    {
	if ((buffer [i] == '\n') && (i+1 < nb))
	{
	    buffer [i+1] = '\0';
	    fin = true;
	}
    }
    return string (buffer);
}

*/

int Socket::write (string str)
{
    int ret = -1;
    ret = ::write (mSocket, str.c_str(), str.length());
    if (ret == ERREUR) throw EXCEPTION ("criture dans la socket");
    return ret;
}

int Socket::close ()
{
    int ret = -1;
    if (mSocket != SOCKET_CLOSED)
    {
        ret = ::close (mSocket);
        if (ret == -1)
	    throw EXCEPTION ("fermeture socket");
        else
            mSocket = SOCKET_CLOSED;
    }
    return ret;
}

