/*
Copyright (c) 2001 Nicolas Roard, Brice Favre, Marie-Charlotte Nguyen

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
#include "thread.h"

int Thread::start (void* a)
{
	arg (a); // stocke les donnes clients
	int code = pthread_create (&_thread_id, NULL, Thread::entry, this);
	return code;
}

void Thread::run (void* a)
{
	exec (a);
}

void* Thread::entry (void* pThis)
{
	Thread* p = (Thread*) pThis;
	p->run (p->arg());
}

void Thread::join (Thread &t)
{
	pthread_join (t.id(), NULL);
}

/*
void Thread::pause (int sec, int nsec)
{
	struct timespec delay;
	delay.tv_sec = sec;
	delay.tv_nsec = nsec;
	pthread_delay_np (&delay);
}
*/

Mutex::Mutex ()
{
	pthread_mutex_init (&_mutex, NULL);
}

void Mutex::lock ()
{
	pthread_mutex_lock (&_mutex);
}

void Mutex::unlock ()
{
	pthread_mutex_unlock (&_mutex);
}

void Mutex::destroy ()
{
	pthread_mutex_destroy (&_mutex);
}

pthread_mutex_t Mutex::getMutex ()
{
	return _mutex;
}

Semaphore::Semaphore ()
{
	v = 1;
	pthread_cond_init (&cond, NULL);
}

void Semaphore::destroy ()
{
	mutex.destroy();
	pthread_cond_destroy (&cond);
}

int Semaphore::up ()
{
	int value_after_op;

	mutex.lock();
	v++;
	value_after_op = v;
	mutex.unlock();
	pthread_cond_signal (&cond);

	return (value_after_op);
}

int Semaphore::down ()
{
	int value_after_op;

	mutex.lock();
	while (v <= 0)
	{
		mutex.unlock();
		usleep (5000);
		mutex.lock();
	}
	v--;
	mutex.unlock();
	value_after_op = v;

/*
	mutex.lock();
	while (v <= 0)
	{
		pthread_mutex_t adr_mutex = mutex.getMutex();
		cout << "on attends que la condition soit change" << endl;
		pthread_cond_wait (&cond, &adr_mutex);
		cout << "condition change, on refait le test sur v : " << v << endl;
	}
	v--;
	value_after_op = v;
	mutex.unlock();
*/
	return (value_after_op);
}

int Semaphore::value ()
{
	int value_after_op;

	mutex.lock ();
	value_after_op = v;
	mutex.unlock ();

	return (value_after_op);
}

