/*
Copyright (c) 2001 Nicolas Roard, Brice Favre, Marie-Charlotte Nguyen

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef __THREAD_H__
#define __THREAD_H__

#include <pthread.h>
#include <iostream>
#include <unistd.h>

class Thread
{
	public:
		Thread () {}
		int start (void* a);
		pthread_t id() { return _thread_id; }
		static void join (Thread &t);
	//	void pause (int sec, int nsec = 0);
	protected:
		void run (void* a);
		static void* entry (void*);
		virtual void exec (void*) = 0;
		void* arg() const { return _arg; }
		void arg (void* a) { _arg = a; }
	private:
		pthread_t _thread_id;
		void* _arg;
};

class Mutex
{
	public:
		Mutex ();
		void lock ();
		void unlock ();
		void destroy ();
		pthread_mutex_t getMutex ();
	private:
		pthread_mutex_t _mutex;
};

class Semaphore
{
	public:
		Semaphore ();
		void destroy ();
		int up ();
		int down ();
		int value ();
	private:
		int v;
		Mutex mutex;
		pthread_cond_t cond;
};

#endif // __THREAD_H__
