/*
Copyright (c) 2001 Nicolas Roard, Brice Favre, Marie-Charlotte Nguyen

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
#include "webthread.h"
#include <fstream>

void getContentType (string& str, string& file)
{

	/*
		NOTE : on appelle le programme /usr/bin/file, qui nous retourne le content-type
		du fichier. Ceci est une mesure temporaire, il faut utiliser la libmime
		pour dterminer nous-mme le content-type.
	*/

	char* arguments [4];
	int p [2];

	if (pipe (p) < 0)
	{
		perror ("pipe chou");
		exit (-1);
	}
	
	arguments [0] = "file";
	arguments [1] = "-bi";
	arguments [2] = new char [file.length()];
	strcpy (arguments [2], file.c_str());
	arguments [3] = NULL;
	
	if (fork() == 0)
	{
		close (p[0]);
		dup2 (p[1], 1); // on dup2 sur stdout
		close (p[1]);
		execvp ("file", arguments);
	}
	else
	{
		close (p[1]);
		char* buf  = new char [256];
		read (p[0], buf, 256);
		bool fin = false;
		for (int i= 0; (!fin) && (i< 256); i++)
		{
			if ((buf[i] == '\n') || (buf [i] == '\r'))
			{
				buf[i] = '\0';
				fin = true;
			}
		}
		str = buf;
		close (p[0]);
	}
}
	

string getFile (string str)
{
	string ret;
	if (str[str.length()-1] == '/') // on demande la page par dfaut
	{
		bool fin = false;
		for (StrList::iterator iter = ServerDefaults.begin (); 
			(!fin) || (iter != ServerDefaults.end()); iter++)
		{
			string file = *iter;
			ifstream in (file.c_str());
			if (in) // si le fichier existe
			{
				ret = ServerRoot + str + file;
				fin = true;
			}
		}
	}
	else
	{
		if (str[0] == '/') // Si on est en relatif
		{
			/*
				Berk... ce hack n'est pas terrible et n'est l que pour tester
				le serveur tant que je n'ai pas incorpor la classe URI qui fera
				tout le sale boulot... ;-)
			*/
			string tmp = str.substr (1,str.size());
			ret = ServerRoot + tmp;
		}
	}
	return ret;
}

void replaceValues (string& str, string var, string value)
{
	int p = str.find (var);
	str.replace (p, var.size(), value);
}
	
void WebThread::exec (void*)
{
	cout << "gestion client\n";
	Socket sd = *(Socket*)arg();
	SemSocket.up ();
	
	string ret = sd.readln();
	string file;

	/*
		NOTE: il faut que je prenne en considration le cas suivant : 
		coupure de la connection (socket) avant d'avoir pu lire le
		header --> actuellement, le code tourne en boucle... 
		--> voir galement pour mettre un thread moniteur pour tuer
		les threads non termins au bout d'un certain temps...
	*/

	while (( ret[0] != '\n') && (ret[0] != '\r'))
	{
		if ((ret.substr(0,3) == "GET") || (ret.substr(0,3) == "get"))
		{
			int beginFile = 0;
			int endFile = 0;
			for (int i = 3; i < (int)ret.length(); i++)
			{
				if ((beginFile == 0) && (ret[i] != ' '))
				{
					beginFile = i;
				}
				if ((beginFile != 0) && (endFile == 0) && (ret[i] == ' '))
				{
					endFile = i;
				}
			}
			file = ret.substr (beginFile, endFile - beginFile);
		}
		cerr << "on a lu : " << ret << endl;
		ret = sd.readln ();
	}

	string query = file;
	
	file = getFile (file);

	cerr << "on veut envoyer le fichier : <" << file << "> " << endl;

	ifstream in (file.c_str());
	string header;
	string msg;
	string ContentType = "text/html";

	if (!in) // erreur d'ouverture sur le fichier
	{
		// On renvoie une page d'erreur 404
		
		header += "HTTP/1.0 404 Not Found\r\n";
		string tmp = ServerRoot + Server404;
		
		cout << "on essaie d'ouvrir la page d'erreur : " << tmp << endl;
		ifstream in404 (tmp.c_str());
		
		if (!in404) // Impossible d'ouvrir la page 404 indique, on renvoie la page par dfaut.
		{
			msg += "<!DOCTYPE HTML PUBLIC \"-//IETF//DTD HTML 2.0//EN\">";
			msg += "<HTML><HEAD>";
			msg += "<TITLE>404 Not Found</TITLE>";
			msg += "</HEAD><BODY>";
			msg += "<H1>Not Found</H1>";
			msg += "The requested URL " + file + " was not found on this server.<P>";
			msg += "<HR>";
			msg += "<ADDRESS>" + ServerName + " at " + HostName + " Port " + ServerPort + "</ADDRESS>";
			msg += "</BODY></HTML>";
		}
		else // On renvoie la page d'erreur 404 indique
		{
			char ch;
			while (in404.get(ch))
			{
				msg += ch;
			}
			replaceValues (msg, "$WEBSERVER_NAME", ServerName);
			replaceValues (msg, "$WEBSERVER_VERSION", ServerVersion);
			replaceValues (msg, "$WEBSERVER_QUERY", query);
		}
	}
	else // le fichier a bien pu tre ouvert, on l'envoie.
	{
		char ch;
		while (in.get(ch))
		{
			msg += ch;
		}
		header += "HTTP/1.0 200 OK\r\n";
		getContentType (ContentType, file);
		//header += "Content-Length: " + itoa(msg.length()) + "\r\n";
	}

	header += "Server: " + ServerName + "\r\n";
	header += "Connection: close\r\n";
	header += "Content-Type: " + ContentType + "\r\n\r\n";
	cerr << "On renvoie le content-type " << ContentType << " pour le fichier " << file << endl;
		
	sd.write(header); // On crit le header HTTP 1.0
	sd.write(msg); // On envoie le fichier.

	cerr << "on ferme la socket" << endl;
	sd.close ();
	pthread_exit (0);
}
